#!/usr/bin/env python
## BEGIN_IMPORT
from pywargame.vassal import SaveIO
from pywargame import version_string
## END_IMPORT

# ====================================================================
def dumpMain():
    from argparse import ArgumentParser 

    ap = ArgumentParser(description='Dump VASSAL save or log')
    ap.add_argument('input',type=str,help='Input save')
    ap.add_argument('-m','--meta',action='store_true',
                    help='Also show metadata')
    ap.add_argument('-n','--line-numbers',action='store_true',
                    help='Prefix each line with a line number')
    ap.add_argument('--version',action='version',version=version_string)
    
    args = ap.parse_args()

    SaveIO.dumpSave(args.input,
                    alsometa=args.meta,
                    linenumbers=args.line_numbers)


# ====================================================================
if __name__ == '__main__':
    dumpMain()
#
#
#

    

    



#!/usr/bin/env python
## BEGIN_IMPORTS
from pywargame.common                 import Verbose
from pywargame.cyberboard.exporter    import GSNExporter
from pywargame.cyberboard.scenario    import Scenario
from pywargame.cyberboard.extractor   import GSNExtractor
from pywargame.cyberboard.head        import GBXHeader
from pywargame                        import version_string
## END_IMPORTS

# ====================================================================
def exportMain():
    
    from argparse import ArgumentParser, FileType, \
         RawDescriptionHelpFormatter 
    from textwrap import wrap, dedent
    from pathlib import Path

    w = lambda s : '\n\n'.join(['\n'.join(wrap(p))
                                for p in dedent(s).split('\n\n')])
    e = '''
           The script can only handle CyberBoard game boxes and scenarios
           from version 3 and up.  Use CBDesign.exe and CBPlay.exe to
           upgrade any pre-version 3 '.gbx' and '.gsn' files, respectively.

           You can override the game box files searched for by passing
           that game box file name to the option '--gamebox'.

           The input file may either be a Cyberboard scenarion (.gsn)
           file, or a zip archive previously generate by
           "gsnextract.py", and, possibly, edited by you.  This can be
           useful if you have a particularly complicated map or
           similar.

           If the script finds the wrong main map, try passing the name
           of the main board as argument to the option '--main-board'.

           If the script adds boards (maps) to the 'Charts' menu,
           rather than as maps, pass a regular expression to match
           these board names, as argument to the option '--map-regex'.
    
           Pass a Python script with the option -p to do
           user post-processing with that script.  Use the
           pywargame VASSAL API to do all sorts of manipulations.
    
           Game boxes with particularly large or complicated maps may
           cause some problems when converting from an internal SVG
           representation to a PNG image.  If you experience that
           problem, the root cause may be an older version of InkScape
           or ImageMagick being used.  Please check that those are
           up-to-date.
           '''
    
    ap = ArgumentParser(description='Create draft VASSAL module',
                        formatter_class= RawDescriptionHelpFormatter,
                        epilog=w(e))
    ap.add_argument('gsnfile',
                    help='The GSN file to data from',
                    type=FileType('rb'))
    ap.add_argument('-p','--patch',
                    help='A python script to patch generated module',
                    type=FileType('r'))
    ap.add_argument('-o','--output',
                    help='Output file to write module to',
                    type=str,
                    default='')
    ap.add_argument('-t','--title',
                    help='Override title',
                    type=str,
                    default=None)
    ap.add_argument('-v','--version',
                    help='Override version',
                    type=str,
                    default=None)
    ap.add_argument('-r','--rules',
                    help='Rules PDF file',
                    type=FileType('r'))
    ap.add_argument('-T','--tutorial',
                    help='Tutorial (v)log file',
                    type=FileType('r'))
    ap.add_argument('-W','--vassal-version',
                    help='Vassal version number',
                    type=str,
                    default='3.7.0')
    ap.add_argument('-V','--verbose',
                    help='Be verbose',
                    action='store_true')
    ap.add_argument('-G','--visible-grids',
                    action='store_true',
                    help='Make grids visible in the module')
    ap.add_argument('-X','--gamebox',
                    type=str,
                    default=None,
                    help='Override gamebox (.gbx) file')
    ap.add_argument('-M','--main-board',
                    type=str,
                    default=None,
                    help='Set the main board')
    ap.add_argument('-R','--map-regex',
                    type=str,
                    default=None,
                    help='Regular expression to match additional maps')
    ap.add_argument('-S','--scenario',
                    action='store_true',
                    help='Make scenario rather than at-start placements')
    ap.add_argument('--pywargame-version',action='version',version=version_string)
    
    args = ap.parse_args()

    gsnname   = args.gsnfile.name
    zipfile   = None
    vmodname  = args.output
    rulesname = args.rules.name    if args.rules    is not None else None
    tutname   = args.tutorial.name if args.tutorial is not None else None
    # Read first 4 bytes from input to see if we're dealing with a GSN file
    magic     = args.gsnfile.read(4)
    args.gsnfile.close()
    if magic != GBXHeader.SCENARIO.encode():
        if magic[:2] == 'PK'.encode():
            from zipfile import ZipFile
            zipfile = ZipFile(gsnname,'r')
    # args.output.close()

    if vmodname == '':
        p        = Path(gsnname)
        vmodname = p.stem+'.vmod'
    
    patchname = args.patch.name if args.patch is not None else None
    if args.patch is not None:
        args.patch.close()

    Verbose().setVerbose(args.verbose)
        
    try:
        scenario  = None
        if zipfile is None:
            scenario = Scenario.fromFile(gsnname,args.gamebox)
        extractor = GSNExtractor(scenario,zipfile)
        exporter  = GSNExporter(extractor._d,
                                title         = args.title,
                                version       = args.version,
                                rules         = rulesname,
                                tutorial      = tutname,
                                visible       = args.visible_grids,
                                main_board    = args.main_board,
                                map_regex     = args.map_regex,
                                vassalVersion = args.vassal_version,
                                do_scenario   = args.scenario)
        exporter.run(vmodname,patchname)
    except Exception as e:
        from sys import stderr 
        print(f'Failed to build {vmodname}: {e}',file=stderr)
        from os import unlink
        try:
            unlink(vmodname)
        except:
            pass

        raise e


# ====================================================================
if __name__ == '__main__':
    exportMain()

# ====================================================================
#
# EOF
#




'''Top-level module'''
version = '0.5.3'
version_string = 'PyWargame '+version

## BEGIN IMPORTS
import pywargame.common     
import pywargame.vassal     
import pywargame.cyberboard 
import pywargame.zuntzu     
import pywargame.latex      
## END IMPORTS



#!/usr/bin/env python
## BEGIN_IMPORT
from pywargame.vassal import SaveIO
from pywargame import version_string
## END_IMPORT

# ====================================================================
def str2pair(inp,asdict=True):
    inner = inp.split(':')
    if len(inner) == 1:
        inner = inp.split('=')

    return inner

# ====================================================================
def zeroMain():
    from argparse import ArgumentParser,  RawDescriptionHelpFormatter 
    from textwrap import wrap, dedent
    from pathlib import Path

    w = lambda s : '\n\n'.join(['\n'.join(wrap(p))
                                for p in dedent(s).split('\n\n')])
    e = '''
        USER, PASSWORD, and SIDE mappings are given as one or more
        elements in the format

        	user:new-value

        where "new-value" may be the new user name, password, or
        side. For example, to change a user from "alice" to "bob",
        pass

        	--user alice:bob

        If you also want to change user "charlie" to "daniella", then
        pass

        	--user alice:bob charlie:daniella

        or 
    
        	--user alice:bob --user charlie:daniella

        and similar for the "--password" and "--side" options.

        You can use the utility "vsavdump.py" to see the content of a
        Vassal save or log file.
    
        When mapping a user name to a new password or side, new
        effective user name is used.
    
        For example, if you map the user "alice" to the new user
        "bob", and you want to set the password of "bob" to be
        "secret", then you should give options like

        	--user alice:bob --password bob:secret
    
        Of course, if you want to change the of existing user
        "charlie" to "hello", then you simply pass
    
        	--password charlie:hello

        and similar when you are mapping sides.

        The output file, if not specified by the option "--output",
        defaults to the same as in the input filename with "-new"
        attached to the stem of the filename.  For example, if you are
        processing "Example.vsav", then you will get
        "Example-new.vsav"
        '''
    ap = ArgumentParser(description='Zero passwords/key VASSAL save or log',
                        formatter_class= RawDescriptionHelpFormatter,
                        epilog=w(e))
    ap.add_argument('input',type=str,help='Input save')
    ap.add_argument('-u','--user',type=str2pair,nargs='*',action='extend',
                    default=[], help='Map input user name to new user name')
    ap.add_argument('-p','--password',type=str2pair,nargs='*',action='extend',
                    default=[],help='Map new user name to new password')
    ap.add_argument('-s','--side',type=str2pair,nargs='*',action='extend',
                    default=[],help='Map new user name to new side')
    ap.add_argument('-k','--key',type=int,default=None,
                    help='Key to encode output save, in range 0-255')
    ap.add_argument('-o','--output',type=str,default=None,
                    help='Output file name')
    ap.add_argument('-V','--verbose',action='store_true',
                    help='Be verbose')
    ap.add_argument('--version',action='version',version=version_string)
    
    args = ap.parse_args()
    
    player_map = {a[0] : a[1] for a in args.user}
    passwd_map = {a[0] : a[1] for a in args.password}
    side_map   = {a[0] : a[1] for a in args.side}
        
    SaveIO.zeroSave(input      = args.input,
                    output     = args.output,
                    player_map = player_map,
                    passwd_map = passwd_map,
                    side_map   = side_map,
                    newkey     = args.key,
                    verbose    = args.verbose)

# ====================================================================
if __name__ == '__main__':
    zeroMain()
#
#
#

    

    
